@@ -40,6 +40,7 @@ import (
 	"golang.org/x/tools/gopls/internal/util/typesutil"
 	"golang.org/x/tools/internal/aliases"
 	"golang.org/x/tools/internal/event"
+	"golang.org/x/tools/internal/stdlib"
 	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/typeparams"
 	"golang.org/x/tools/internal/typesinternal"
@@ -70,12 +71,17 @@ type hoverJSON struct {
 
 	// LinkPath is the pkg.go.dev link for the given symbol.
 	// For example, the "go/ast" part of "pkg.go.dev/go/ast#Node".
+	// It may have a module version suffix "@v1.2.3".
 	LinkPath string `json:"linkPath"`
 
 	// LinkAnchor is the pkg.go.dev link anchor for the given symbol.
 	// For example, the "Node" part of "pkg.go.dev/go/ast#Node".
 	LinkAnchor string `json:"linkAnchor"`
 
+	// stdVersion is the Go release version at which this symbol became available.
+	// It is nil for non-std library.
+	stdVersion *stdlib.Version
+
 	// New fields go below, and are unexported. The existing
 	// exported fields are underspecified and have already
 	// constrained our movements too much. A detailed JSON
@@ -98,7 +104,10 @@ type hoverJSON struct {
 }
 
 // Hover implements the "textDocument/hover" RPC for Go files.
-func Hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, position protocol.Position) (*protocol.Hover, error) {
+// It may return nil even on success.
+//
+// If pkgURL is non-nil, it should be used to generate doc links.
+func Hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, position protocol.Position, pkgURL func(path PackagePath, fragment string) protocol.URI) (*protocol.Hover, error) {
 	ctx, done := event.Start(ctx, "golang.Hover")
 	defer done()
 
@@ -109,7 +118,7 @@ func Hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, positi
 	if h == nil {
 		return nil, nil
 	}
-	hover, err := formatHover(h, snapshot.Options())
+	hover, err := formatHover(h, snapshot.Options(), pkgURL)
 	if err != nil {
 		return nil, err
 	}
@@ -125,7 +134,44 @@ func Hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, positi
 // hover computes hover information at the given position. If we do not support
 // hovering at the position, it returns _, nil, nil: an error is only returned
 // if the position is valid but we fail to compute hover information.
+//
+// TODO(adonovan): strength-reduce file.Handle to protocol.DocumentURI.
 func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp protocol.Position) (protocol.Range, *hoverJSON, error) {
+	// Check for hover inside the builtin file before attempting type checking
+	// below. NarrowestPackageForFile may or may not succeed, depending on
+	// whether this is a GOROOT view, but even if it does succeed the resulting
+	// package will be command-line-arguments package. The user should get a
+	// hover for the builtin object, not the object type checked from the
+	// builtin.go.
+	if snapshot.IsBuiltin(fh.URI()) {
+		pgf, err := snapshot.BuiltinFile(ctx)
+		if err != nil {
+			return protocol.Range{}, nil, err
+		}
+		pos, err := pgf.PositionPos(pp)
+		if err != nil {
+			return protocol.Range{}, nil, err
+		}
+		path, _ := astutil.PathEnclosingInterval(pgf.File, pos, pos)
+		if id, ok := path[0].(*ast.Ident); ok {
+			rng, err := pgf.NodeRange(id)
+			if err != nil {
+				return protocol.Range{}, nil, err
+			}
+			var obj types.Object
+			if id.Name == "Error" {
+				obj = types.Universe.Lookup("error").Type().Underlying().(*types.Interface).Method(0)
+			} else {
+				obj = types.Universe.Lookup(id.Name)
+			}
+			if obj != nil {
+				h, err := hoverBuiltin(ctx, snapshot, obj)
+				return rng, h, err
+			}
+		}
+		return protocol.Range{}, nil, nil // no object to hover
+	}
+
 	pkg, pgf, err := NarrowestPackageForFile(ctx, snapshot, fh.URI())
 	if err != nil {
 		return protocol.Range{}, nil, err
@@ -196,7 +242,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 			if hoverRange == nil {
 				hoverRange = &rng
 			}
-			return *hoverRange, hoverJSON, nil
+			return *hoverRange, hoverJSON, nil // (hoverJSON may be nil)
 		}
 	}
 	// Handle hovering over (non-import-path) literals.
@@ -533,13 +579,13 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 			pkg := obj.Pkg()
 			if recv != nil {
 				linkName = fmt.Sprintf("(%s.%s).%s", pkg.Name(), recv.Name(), obj.Name())
-				if obj.Exported() && recv.Exported() && pkg.Scope().Lookup(recv.Name()) == recv {
+				if obj.Exported() && recv.Exported() && isPackageLevel(recv) {
 					linkPath = pkg.Path()
 					anchor = fmt.Sprintf("%s.%s", recv.Name(), obj.Name())
 				}
 			} else {
 				linkName = fmt.Sprintf("%s.%s", pkg.Name(), obj.Name())
-				if obj.Exported() && pkg.Scope().Lookup(obj.Name()) == obj {
+				if obj.Exported() && isPackageLevel(obj) {
 					linkPath = pkg.Path()
 					anchor = obj.Name()
 				}
@@ -554,6 +600,11 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 		linkPath = strings.Replace(linkPath, mod.Path, mod.Path+"@"+mod.Version, 1)
 	}
 
+	var version *stdlib.Version
+	if symbol := StdSymbolOf(obj); symbol != nil {
+		version = &symbol.Version
+	}
+
 	return *hoverRange, &hoverJSON{
 		Synopsis:          doc.Synopsis(docText),
 		FullDocumentation: docText,
@@ -565,6 +616,7 @@ func hover(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, pp pro
 		typeDecl:          typeDecl,
 		methods:           methods,
 		promotedFields:    fields,
+		stdVersion:        version,
 	}, nil
 }
 
@@ -586,13 +638,16 @@ func hoverBuiltin(ctx context.Context, snapshot *cache.Snapshot, obj types.Objec
 		}, nil
 	}
 
-	pgf, node, err := builtinDecl(ctx, snapshot, obj)
+	pgf, ident, err := builtinDecl(ctx, snapshot, obj)
 	if err != nil {
 		return nil, err
 	}
 
-	var comment *ast.CommentGroup
-	path, _ := astutil.PathEnclosingInterval(pgf.File, node.Pos(), node.End())
+	var (
+		comment *ast.CommentGroup
+		decl    ast.Decl
+	)
+	path, _ := astutil.PathEnclosingInterval(pgf.File, ident.Pos(), ident.Pos())
 	for _, n := range path {
 		switch n := n.(type) {
 		case *ast.GenDecl:
@@ -600,17 +655,17 @@ func hoverBuiltin(ctx context.Context, snapshot *cache.Snapshot, obj types.Objec
 			comment = n.Doc
 			node2 := *n
 			node2.Doc = nil
-			node = &node2
+			decl = &node2
 		case *ast.FuncDecl:
 			// Ditto.
 			comment = n.Doc
 			node2 := *n
 			node2.Doc = nil
-			node = &node2
+			decl = &node2
 		}
 	}
 
-	signature := formatNodeFile(pgf.Tok, node)
+	signature := formatNodeFile(pgf.Tok, decl)
 	// Replace fake types with their common equivalent.
 	// TODO(rfindley): we should instead use obj.Type(), which would have the
 	// *actual* types of the builtin call.
@@ -1086,7 +1141,8 @@ func parseFull(ctx context.Context, snapshot *cache.Snapshot, fset *token.FileSe
 	return pgf, fullPos, nil
 }
 
-func formatHover(h *hoverJSON, options *settings.Options) (string, error) {
+// If pkgURL is non-nil, it should be used to generate doc links.
+func formatHover(h *hoverJSON, options *settings.Options, pkgURL func(path PackagePath, fragment string) protocol.URI) (string, error) {
 	maybeMarkdown := func(s string) string {
 		if s != "" && options.PreferredContentFormat == protocol.Markdown {
 			s = fmt.Sprintf("```go\n%s\n```", strings.Trim(s, "\n"))
@@ -1121,11 +1177,15 @@ func formatHover(h *hoverJSON, options *settings.Options) (string, error) {
 			formatDoc(h, options),
 			maybeMarkdown(h.promotedFields),
 			maybeMarkdown(h.methods),
-			formatLink(h, options),
+			fmt.Sprintf("Added in %v", h.stdVersion),
+			formatLink(h, options, pkgURL),
 		}
 		if h.typeDecl != "" {
 			parts[0] = "" // type: suppress redundant Signature
 		}
+		if h.stdVersion == nil || *h.stdVersion == stdlib.Version(0) {
+			parts[5] = "" // suppress stdlib version if not applicable or initial version 1.0
+		}
 		parts = slices.Remove(parts, "")
 
 		var b strings.Builder
@@ -1146,18 +1206,102 @@ func formatHover(h *hoverJSON, options *settings.Options) (string, error) {
 	}
 }
 
-func formatLink(h *hoverJSON, options *settings.Options) string {
-	if !options.LinksInHover || options.LinkTarget == "" || h.LinkPath == "" {
+// StdSymbolOf returns the std lib symbol information of the given obj.
+// It returns nil if the input obj is not an exported standard library symbol.
+func StdSymbolOf(obj types.Object) *stdlib.Symbol {
+	if !obj.Exported() || obj.Pkg() == nil {
+		return nil
+	}
+
+	// Symbols that not defined in standard library should return early.
+	// TODO(hxjiang): The returned slices is binary searchable.
+	symbols := stdlib.PackageSymbols[obj.Pkg().Path()]
+	if symbols == nil {
+		return nil
+	}
+
+	// Handle Function, Type, Const & Var.
+	if isPackageLevel(obj) {
+		for _, s := range symbols {
+			if s.Kind == stdlib.Method || s.Kind == stdlib.Field {
+				continue
+			}
+			if s.Name == obj.Name() {
+				return &s
+			}
+		}
+		return nil
+	}
+
+	// Handle Method.
+	if fn, _ := obj.(*types.Func); fn != nil {
+		isPtr, named := typesinternal.ReceiverNamed(fn.Type().(*types.Signature).Recv())
+		if isPackageLevel(named.Obj()) {
+			for _, s := range symbols {
+				if s.Kind != stdlib.Method {
+					continue
+				}
+				ptr, recv, name := s.SplitMethod()
+				if ptr == isPtr && recv == named.Obj().Name() && name == fn.Name() {
+					return &s
+				}
+			}
+			return nil
+		}
+	}
+
+	// Handle Field.
+	if v, _ := obj.(*types.Var); v != nil && v.IsField() {
+		for _, s := range symbols {
+			if s.Kind != stdlib.Field {
+				continue
+			}
+
+			typeName, fieldName := s.SplitField()
+			if fieldName != v.Name() {
+				continue
+			}
+
+			typeObj := obj.Pkg().Scope().Lookup(typeName)
+			if typeObj == nil {
+				continue
+			}
+
+			if fieldObj, _, _ := types.LookupFieldOrMethod(typeObj.Type(), true, obj.Pkg(), fieldName); obj == fieldObj {
+				return &s
+			}
+		}
+		return nil
+	}
+
+	return nil
+}
+
+// If pkgURL is non-nil, it should be used to generate doc links.
+func formatLink(h *hoverJSON, options *settings.Options, pkgURL func(path PackagePath, fragment string) protocol.URI) string {
+	if options.LinksInHover == false || h.LinkPath == "" {
 		return ""
 	}
-	plainLink := cache.BuildLink(options.LinkTarget, h.LinkPath, h.LinkAnchor)
+	var url protocol.URI
+	var caption string
+	if pkgURL != nil { // LinksInHover == "gopls"
+		path, _, _ := strings.Cut(h.LinkPath, "@") // remove optional module version suffix
+		url = pkgURL(PackagePath(path), h.LinkAnchor)
+		caption = "in gopls doc viewer"
+	} else {
+		if options.LinkTarget == "" {
+			return ""
+		}
+		url = cache.BuildLink(options.LinkTarget, h.LinkPath, h.LinkAnchor)
+		caption = "on " + options.LinkTarget
+	}
 	switch options.PreferredContentFormat {
 	case protocol.Markdown:
-		return fmt.Sprintf("[`%s` on %s](%s)", h.SymbolName, options.LinkTarget, plainLink)
+		return fmt.Sprintf("[`%s` %s](%s)", h.SymbolName, caption, url)
 	case protocol.PlainText:
 		return ""
 	default:
-		return plainLink
+		return url
 	}
 }
 
