@@ -10,8 +10,8 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"go/ast"
 	"io"
+	"log"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -93,18 +93,8 @@ func (*commandHandler) AddTelemetryCounters(_ context.Context, args command.AddT
 
 // commandConfig configures common command set-up and execution.
 type commandConfig struct {
-	// TODO(adonovan): whether a command is synchronous or
-	// asynchronous is part of the server interface contract, not
-	// a mere implementation detail of the handler.
-	// Export a (command.Command).IsAsync() property so that
-	// clients can tell. (The tricky part is ensuring the handler
-	// remains consistent with the command.Command metadata, as at
-	// the point were we read the 'async' field below, we no
-	// longer know that command.Command.)
-
-	async       bool                 // whether to run the command asynchronously. Async commands can only return errors.
 	requireSave bool                 // whether all files must be saved for the command to work
-	progress    string               // title to use for progress reporting. If empty, no progress will be reported.
+	progress    string               // title to use for progress reporting. If empty, no progress will be reported. Mandatory for async commands.
 	forView     string               // view to resolve to a snapshot; incompatible with forURI
 	forURI      protocol.DocumentURI // URI to resolve to a snapshot. If unset, snapshot will be nil.
 }
@@ -113,9 +103,9 @@ type commandConfig struct {
 // be populated, depending on which configuration is set. See comments in-line
 // for details.
 type commandDeps struct {
-	snapshot *cache.Snapshot    // present if cfg.forURI was set
+	snapshot *cache.Snapshot    // present if cfg.forURI or forView was set
 	fh       file.Handle        // present if cfg.forURI was set
-	work     *progress.WorkDone // present cfg.progress was set
+	work     *progress.WorkDone // present if cfg.progress was set
 }
 
 type commandFunc func(context.Context, commandDeps) error
@@ -195,7 +185,12 @@ func (c *commandHandler) run(ctx context.Context, cfg commandConfig, run command
 		}
 		return err
 	}
-	if cfg.async {
+
+	if enum := command.Command(c.params.Command); enum.IsAsync() {
+		if cfg.progress == "" {
+			log.Fatalf("asynchronous command %q does not enable progress reporting",
+				enum)
+		}
 		go func() {
 			if err := runcmd(); err != nil {
 				showMessage(ctx, c.s.client, protocol.Error, err.Error())
@@ -491,6 +486,7 @@ func dropDependency(pm *cache.ParsedModule, modulePath string) ([]protocol.TextE
 	return protocol.EditsFromDiffEdits(pm.Mapper, diff)
 }
 
+// Test is an alias for RunTests (with splayed arguments).
 func (c *commandHandler) Test(ctx context.Context, uri protocol.DocumentURI, tests, benchmarks []string) error {
 	return c.RunTests(ctx, command.RunTestsArgs{
 		URI:        uri,
@@ -508,12 +504,9 @@ func (c *commandHandler) Doc(ctx context.Context, loc protocol.Location) error {
 		if err != nil {
 			return err
 		}
-
-		// When invoked from a _test.go file, show the
-		// documentation of the package under test.
-		pkgpath := pkg.Metadata().PkgPath
-		if pkg.Metadata().ForTest != "" {
-			pkgpath = pkg.Metadata().ForTest
+		start, end, err := pgf.RangePos(loc.Range)
+		if err != nil {
+			return err
 		}
 
 		// Start web server.
@@ -522,59 +515,12 @@ func (c *commandHandler) Doc(ctx context.Context, loc protocol.Location) error {
 			return err
 		}
 
-		// Compute fragment (e.g. "#Buffer.Len") based on
-		// enclosing top-level declaration, if exported.
-		var fragment string
-		pos, err := pgf.PositionPos(loc.Range.Start)
-		if err != nil {
-			return err
-		}
-		path, _ := astutil.PathEnclosingInterval(pgf.File, pos, pos)
-		if n := len(path); n > 1 {
-			switch decl := path[n-2].(type) {
-			case *ast.FuncDecl:
-				if decl.Name.IsExported() {
-					// e.g. "#Println"
-					fragment = decl.Name.Name
-
-					// method?
-					if decl.Recv != nil && len(decl.Recv.List) > 0 {
-						recv := decl.Recv.List[0].Type
-						if star, ok := recv.(*ast.StarExpr); ok {
-							recv = star.X // *N -> N
-						}
-						if id, ok := recv.(*ast.Ident); ok && id.IsExported() {
-							// e.g. "#Buffer.Len"
-							fragment = id.Name + "." + fragment
-						} else {
-							fragment = ""
-						}
-					}
-				}
-
-			case *ast.GenDecl:
-				// path=[... Spec? GenDecl File]
-				for _, spec := range decl.Specs {
-					if n > 2 && spec == path[n-3] {
-						var name *ast.Ident
-						switch spec := spec.(type) {
-						case *ast.ValueSpec:
-							// var, const: use first name
-							name = spec.Names[0]
-						case *ast.TypeSpec:
-							name = spec.Name
-						}
-						if name != nil && name.IsExported() {
-							fragment = name.Name
-						}
-						break
-					}
-				}
-			}
-		}
+		// Compute package path and optional symbol fragment
+		// (e.g. "#Buffer.Len") from the the selection.
+		pkgpath, fragment, _ := golang.DocFragment(pkg, pgf, start, end)
 
 		// Direct the client to open the /pkg page.
-		url := web.pkgURL(deps.snapshot.View(), pkgpath, fragment)
+		url := web.PkgURL(deps.snapshot.View().ID(), pkgpath, fragment)
 		openClientBrowser(ctx, c.s.client, url)
 
 		return nil
@@ -583,9 +529,8 @@ func (c *commandHandler) Doc(ctx context.Context, loc protocol.Location) error {
 
 func (c *commandHandler) RunTests(ctx context.Context, args command.RunTestsArgs) error {
 	return c.run(ctx, commandConfig{
-		async:       true,
-		progress:    "Running go test",
-		requireSave: true, // go test honors overlays, but tests themselves cannot
+		progress:    "Running go test", // (asynchronous)
+		requireSave: true,              // go test honors overlays, but tests themselves cannot
 		forURI:      args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
 		return c.runTests(ctx, deps.snapshot, deps.work, args.URI, args.Tests, args.Benchmarks)
@@ -995,6 +940,26 @@ func (c *commandHandler) AddImport(ctx context.Context, args command.AddImportAr
 	})
 }
 
+func (c *commandHandler) ExtractToNewFile(ctx context.Context, args protocol.Location) error {
+	return c.run(ctx, commandConfig{
+		progress: "Extract to a new file",
+		forURI:   args.URI,
+	}, func(ctx context.Context, deps commandDeps) error {
+		edit, err := golang.ExtractToNewFile(ctx, deps.snapshot, deps.fh, args.Range)
+		if err != nil {
+			return err
+		}
+		resp, err := c.s.client.ApplyEdit(ctx, &protocol.ApplyWorkspaceEditParams{Edit: *edit})
+		if err != nil {
+			return fmt.Errorf("could not apply edits: %v", err)
+		}
+		if !resp.Applied {
+			return fmt.Errorf("edits not applied: %s", resp.FailureReason)
+		}
+		return nil
+	})
+}
+
 func (c *commandHandler) StartDebugging(ctx context.Context, args command.DebuggingArgs) (result command.DebuggingResult, _ error) {
 	addr := args.Addr
 	if addr == "" {
@@ -1088,9 +1053,8 @@ func (c *commandHandler) RunGovulncheck(ctx context.Context, args command.Vulnch
 	// synchronize the start of the run and return the token.
 	tokenChan := make(chan protocol.ProgressToken, 1)
 	err := c.run(ctx, commandConfig{
-		async:       true, // need to be async to be cancellable
-		progress:    "govulncheck",
-		requireSave: true, // govulncheck cannot honor overlays
+		progress:    "govulncheck", // (asynchronous)
+		requireSave: true,          // govulncheck cannot honor overlays
 		forURI:      args.URI,
 	}, func(ctx context.Context, deps commandDeps) error {
 		tokenChan <- deps.work.Token()
@@ -1470,19 +1434,29 @@ func (c *commandHandler) Views(ctx context.Context) ([]command.View, error) {
 	return summaries, nil
 }
 
-func (c *commandHandler) FreeSymbols(ctx context.Context, uri protocol.DocumentURI, rng protocol.Range) error {
-	return c.run(ctx, commandConfig{
-		forURI: uri,
-	}, func(ctx context.Context, deps commandDeps) error {
-		web, err := c.s.getWeb()
-		if err != nil {
-			return err
-		}
-		url := web.freesymbolsURL(deps.snapshot.View(), protocol.Location{
-			URI:   deps.fh.URI(),
-			Range: rng,
-		})
-		openClientBrowser(ctx, c.s.client, url)
-		return nil
-	})
+func (c *commandHandler) FreeSymbols(ctx context.Context, viewID string, loc protocol.Location) error {
+	web, err := c.s.getWeb()
+	if err != nil {
+		return err
+	}
+	url := web.freesymbolsURL(viewID, loc)
+	openClientBrowser(ctx, c.s.client, url)
+	return nil
+}
+
+func (c *commandHandler) Assembly(ctx context.Context, viewID, packageID, symbol string) error {
+	web, err := c.s.getWeb()
+	if err != nil {
+		return err
+	}
+	url := web.assemblyURL(viewID, packageID, symbol)
+	openClientBrowser(ctx, c.s.client, url)
+	return nil
+}
+
+func (c *commandHandler) ScanImports(ctx context.Context) error {
+	for _, v := range c.s.session.Views() {
+		v.ScanImports()
+	}
+	return nil
 }
