@@ -21,11 +21,13 @@ var (
 // DefaultOptions is the options that are used for Gopls execution independent
 // of any externally provided configuration (LSP initialization, command
 // invocation, etc.).
+//
+// It is the source from which gopls/doc/settings.md is generated.
 func DefaultOptions(overrides ...func(*Options)) *Options {
 	optionsOnce.Do(func() {
 		var commands []string
 		for _, c := range command.Commands {
-			commands = append(commands, c.ID())
+			commands = append(commands, c.String())
 		}
 		defaultOptions = &Options{
 			ClientOptions: ClientOptions{
@@ -47,7 +49,10 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 						protocol.RefactorRewrite:       true,
 						protocol.RefactorInline:        true,
 						protocol.RefactorExtract:       true,
-						protocol.GoDoc:                 true,
+						GoAssembly:                     true,
+						GoDoc:                          true,
+						GoFreeSymbols:                  true,
+						// Not GoTest: it must be explicit in CodeActionParams.Context.Only
 					},
 					file.Mod: {
 						protocol.SourceOrganizeImports: true,
@@ -97,14 +102,14 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 						ExperimentalPostfixCompletions: true,
 						CompleteFunctionCalls:          true,
 					},
-					Codelenses: map[string]bool{
-						string(command.Generate):          true,
-						string(command.RegenerateCgo):     true,
-						string(command.Tidy):              true,
-						string(command.GCDetails):         false,
-						string(command.UpgradeDependency): true,
-						string(command.Vendor):            true,
-						// TODO(hyangah): enable command.RunGovulncheck.
+					Codelenses: map[CodeLensSource]bool{
+						CodeLensGenerate:          true,
+						CodeLensRegenerateCgo:     true,
+						CodeLensTidy:              true,
+						CodeLensGCDetails:         false,
+						CodeLensUpgradeDependency: true,
+						CodeLensVendor:            true,
+						CodeLensRunGovulncheck:    false, // TODO(hyangah): enable
 					},
 				},
 			},
@@ -119,11 +124,6 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 				IncludeReplaceInWorkspace:   false,
 				ZeroConfig:                  true,
 			},
-			Hooks: Hooks{
-				URLRegexp:            urlRegexp(),
-				DefaultAnalyzers:     analyzers(),
-				StaticcheckAnalyzers: map[string]*Analyzer{},
-			},
 		}
 	})
 	options := defaultOptions.Clone()
